"""PyToast: the one only cross-platform Toaster/Notification in python """

import os
import platform

__all__ = ['Notification', 'NotificationError']

#env\Scripts\activate

class NotificationError(Exception):
    """Exception raised when a Notification Error happens"""
    def __init__(self):
        pass

class Notif:
    """
        Makes toast for every major os Windows, Mac & linux

        >Send toast on Windows https://stackoverflow.com/questions/17651017/python-post-osx-notification
        >Send toast on linux https://github.com/YuriyLisovskiy/pynotifier
        >Send toast on MacOS https://stackoverflow.com/questions/17651017/python-post-osx-notification
        
        ✔️ = Working
        ❌ = Not Working
        ❔ = I Dont Know

        Windows 10 = ✔️  MacOS Catalina = ❔     Ubuntu = ❔
        Windows 8.1 = ❔  MacOS Mojave = ✔️      Debian = ❔
        Windows 7 = ❔    MacOS High Sierra = ❔  CentOS = ❔

        || Send Testing for new OS or OS version to the issuse tab plz

        Parameters
            ----------
            title : str
                title that the toast should display
            message : str
                message that the toast should display
            duration : str , (optional)
                the amount in seconds that toast show be visible
                defaults to 10 seconds  
            urgency : str , (optional) (linux)
                defaults to low
            icon-path : str, (optional) (linux, windows)
                the icon to show on the toast
                image must be 
                    .ico for windows
                    .png for linux

        Returns
            ----------
            True : Boolean (Implementation Soon)
                returns 'True' if an error occured
            False : Boolean (Implementation Soon)
                returns 'False' if an error has not occured

        """
    def __init__(self, title, message, duration=10, urgency="low", icon_path="python.ico"):
        self.title = title
        self.message = message
        self.duration = duration
        self.urgency = urgency
        self.icon_path = icon_path

    def __repr__(self):
        return f"{self.message}"

    def __str__(self):
        return f"{self.message}"

    def Toast(self) -> bool:
        try:
            if platform.system().lower().startswith('win'): 
                """
                Windows NT
                """
                try:
                    try:
                        from win10toast import ToastNotifier
                    except ImportError as e:
                        print("Installing win10toast")
                        os.system('pip install win10toast')
                    finally:
                        from win10toast import ToastNotifier

                    toaster = ToastNotifier()
                    toaster.show_toast(self.title, self.message, icon_path=self.icon_path, duration=self.duration)
                    return True

                except BaseException as e:
                    # raise NotificationError
                    print("Cookie: Windows Toast Error: {}".format(e))
                    return False

            elif platform.system().lower().startswith('lin'):
                """
                Linux
                """
                URGENCY_LOW = 'low'
                URGENCY_NORMAL = 'normal'
                URGENCY_CRITICAL = 'critical'
                try:
                    import subprocess
                    if self.urgency not in [URGENCY_LOW, URGENCY_NORMAL, URGENCY_CRITICAL]:
                        print('Cookie: invalid urgency was given: {}').format(self.urgency)
                        urgency = "low"

                    command = [
                        'notify-send', '{}'.format(self.title),
                        '{}'.format(self.message),
                        '-u', self.urgency,
                        '-t', '{}'.format(self.duration * 1000)
                    ]
                    if icon_path is not None:
                        command += ['-i', self.icon_path]
                    subprocess.call(command)
                    return True
                except BaseException as e:
                    print("Cookie: Linux Toast Error: {}".format(e))
                    # raise NotificationError
                    return False

            elif platform.system().lower().startswith('dar'):
                """
                Darwin MacOS
                """
                try:
                    command = f""" osascript -e 'display notification "{self.message}" with title "{self.title}"' """
                    os.system(command)
                    return True
                except BaseException as e:
                    print("Cookie: MacOS Toast Error: {}".format(e))
                    # raise NotificationError
                    return False
            else:
                raise SystemError(f'Notifications are not supported for {platform.system()} system')

        except BaseException as e:
                print("Cookie: Toast error: {}".format(e))
                # raise NotificationError
                return False




try: 
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: 
    # for pip <= 9.0.3
    from pip.req import parse_requirements

from operator import attrgetter
from os import path
from setuptools import setup, find_packages

def read(fname):
    return open(path.join(path.dirname(__file__), fname)).read()

def from_here(relative_path):
    return path.join(path.dirname(__file__), relative_path)

with open("README.md", "r") as fh:
    long_description = fh.read()

# python setup.py sdist bdist_wheel
# python -m twine upload --skip-existing --repository testpypi dist/* 

# python -m twine upload --skip-existing dist/*
# python -m twine upload dist/*

setup(
    name="creed",
    version="0.2",
    install_requires=[
        "pypiwin32==223",
        "pywin32==227",
        "win10toast==0.9"
    ],
    author="Yusuf Ahmed",
    author_email="yusufahmed172@gmail.com",
    description="the one and only cross-platform Toaster/Notification in python",
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    url="https://github.com/yusuf8ahmed/PyToast",
    packages=find_packages(),
        package_data={
        '': ['*.txt'],
        'creed': ['*.ico'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)